@@ -1,6 +1,5 @@
-/* $Id: gucharmap-codepoint-list.h 1493 2007-12-12 19:33:26Z chpe $ */
 /*
- * Copyright (c) 2004 Noah Levitt
+ * Copyright © 2004 Noah Levitt
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,6 +16,10 @@
  * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
  */
 
+#if !defined (__GUCHARMAP_GUCHARMAP_H_INSIDE__) && !defined (GUCHARMAP_COMPILATION)
+#error "Only <gucharmap/gucharmap.h> can be included directly."
+#endif
+
 #ifndef GUCHARMAP_CODEPOINT_LIST_H
 #define GUCHARMAP_CODEPOINT_LIST_H
 
@@ -24,24 +27,23 @@
 
 G_BEGIN_DECLS
 
-#define GUCHARMAP_CODEPOINT_LIST(obj) \
-            (G_TYPE_CHECK_INSTANCE_CAST ((obj), gucharmap_codepoint_list_get_type (), GucharmapCodepointList))
+#define GUCHARMAP_TYPE_CODEPOINT_LIST             (gucharmap_codepoint_list_get_type ())
+#define GUCHARMAP_CODEPOINT_LIST(o)               (G_TYPE_CHECK_INSTANCE_CAST ((o), GUCHARMAP_TYPE_CODEPOINT_LIST, GucharmapCodepointList))
+#define GUCHARMAP_CODEPOINT_LIST_CLASS(k)         (G_TYPE_CHECK_CLASS_CAST((k), GUCHARMAP_TYPE_CODEPOINT_LIST, GucharmapCodepointListClass))
+#define GUCHARMAP_IS_CODEPOINT_LIST(o)            (G_TYPE_CHECK_INSTANCE_TYPE ((o), GUCHARMAP_TYPE_CODEPOINT_LIST))
+#define GUCHARMAP_IS_CODEPOINT_LIST_CLASS(k)      (G_TYPE_CHECK_CLASS_TYPE ((k), GUCHARMAP_TYPE_CODEPOINT_LIST))
+#define GUCHARMAP_CODEPOINT_LIST_GET_CLASS(o)     (G_TYPE_INSTANCE_GET_CLASS ((o), GUCHARMAP_TYPE_CODEPOINT_LIST, GucharmapCodepointListClass))
 
-#define GUCHARMAP_CODEPOINT_LIST_CLASS(clazz) \
-            (G_TYPE_CHECK_CLASS_CAST ((clazz), gucharmap_codepoint_list_get_type (), GucharmapCodepointListClass))
-
-#define IS_GUCHARMAP_CODEPOINT_LIST(obj) \
-             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), gucharmap_codepoint_list_get_type ()))
-
-#define GUCHARMAP_CODEPOINT_LIST_GET_CLASS(obj) \
-             (G_TYPE_INSTANCE_GET_CLASS ((obj), gucharmap_codepoint_list_get_type (), GucharmapCodepointListClass))
-
-typedef struct _GucharmapCodepointList GucharmapCodepointList;
-typedef struct _GucharmapCodepointListClass GucharmapCodepointListClass;
+typedef struct _GucharmapCodepointList        GucharmapCodepointList;
+typedef struct _GucharmapCodepointListPrivate GucharmapCodepointListPrivate;
+typedef struct _GucharmapCodepointListClass   GucharmapCodepointListClass;
 
 struct _GucharmapCodepointList
 {
-  GObject parent;
+  GObject parent_instance;
+
+  /*< private >*/
+  GucharmapCodepointListPrivate *priv;
 };
 
 struct _GucharmapCodepointListClass
@@ -57,15 +59,15 @@ struct _GucharmapCodepointListClass
 };
 
 GType                    gucharmap_codepoint_list_get_type       (void);
-GucharmapCodepointList * gucharmap_codepoint_list_new            (gunichar start,
-                                                                  gunichar end);
-gunichar                 gucharmap_codepoint_list_get_char       (GucharmapCodepointList *list, 
+
+gunichar                 gucharmap_codepoint_list_get_char       (GucharmapCodepointList *list,
                                                                   gint                    index);
+
 gint                     gucharmap_codepoint_list_get_index      (GucharmapCodepointList *list, 
                                                                   gunichar                wc);
+
 gint                     gucharmap_codepoint_list_get_last_index (GucharmapCodepointList *list);
 
 G_END_DECLS
 
 #endif /* #ifndef GUCHARMAP_CODEPOINT_LIST_H */
-
